==> efence_2_4_12.tar.gz: efence/efence.h <==

/*
 * Electric Fence - Red-Zone memory allocator.
 * Copyright (C) 1987-1999 Bruce Perens <bruce@perens.com>
 * Copyright (C) 2002-2005 Hayati Ayguen <h_ayguen@web.de>, Procitec GmbH
 * License: GNU GPL (GNU General Public License, see COPYING-GPL)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * FILE CONTENTS:
 * header file for inclusion from YOUR application code
 */

/*
 * #include <stdlib.h>
 *
 * You must include <stdlib.h> before including <efence.h> !
 *
 */

/* for enabling inclusion of efence.h after inclusion of efencint.h */
/* remove previous definitions */
#undef malloc
#undef calloc
#undef realloc
#undef free
#undef EF_newFrame
#undef EF_delFrame

#undef EF_STACK_PROT_START
#undef EF_STACK_PROT_END

#undef CA_DECLARE
#undef CA_DEFINE
#undef CA_REF

#ifndef _EFENCE_CONFIG_H_
#include "efence_config.h"
#endif

#ifdef EF_NO_EFENCE

/* enable these macros even in release code, but do nothing */
#define EF_newFrame()               do { } while(0)
#define EF_delFrame()               do { } while(0)

#define EF_ASSERT(EXPR)             do { } while(0)

#define CA_DECLARE(NAME,SIZE)       do { } while(0)
#define CA_DEFINE(TYPE,NAME,SIZE)   TYPE NAME[SIZE]
#define CA_REF(NAME,INDEX)          NAME[INDEX]

#else /* ifndef EF_NO_EFENCE */

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _EFENCE_SLOT_STATE
#define _EFENCE_SLOT_STATE
/*
 * Enum _EF_Slot_State indicates the status of a slot
 * Warning: Also have a look at ALLOCATED_MASK in efence.c
 */
enum _EF_Slot_State
{
    EFST_NOT_IN_USE      = 0   /* Available to represent a malloc buffer. */
  , EFST_FREE            = 1   /* A free buffer. */
  , EFST_ALLOC_MALLOC    = 2   /* allocated through C "malloc()", "calloc()", "realloc()" .. */
  , EFST_ALLOC_NEW_ELEM  = 4   /* allocated through C++ "new"   */
  , EFST_ALLOC_NEW_ARRAY = 8   /* allocated through C++ "new[]" */
  , EFST_INTERNAL_USE    = 16  /* A buffer used internally by malloc(). */
  , EFST_PROTECTED       = 32  /* A freed buffer that can not be allocated again. */
};
#endif /* _EFENCE_SLOT_STATE */


#ifndef EF_NO_LEAKDETECTION

void * _eff_malloc(size_t size, enum _EF_Slot_State, const char * filename, int lineno);
void * _eff_calloc(size_t elemCount, size_t elemSize, enum _EF_Slot_State, const char * filename, int lineno);
void * _eff_realloc(void * baseAdr, size_t newSize, enum _EF_Slot_State, const char * filename, int lineno);
void   _eff_free(void * baseAdr, enum _EF_Slot_State);

void  EF_newFrame(void);
void  EF_delFrame(void);

#define malloc(SIZE)                _eff_malloc(SIZE, EFST_ALLOC_MALLOC, __FILE__, __LINE__)
#define calloc(ELEMCOUNT, ELEMSIZE) _eff_calloc(ELEMCOUNT, ELEMSIZE, EFST_ALLOC_MALLOC, __FILE__, __LINE__)
#define realloc(BASEADR, NEWSIZE)   _eff_realloc(BASEADR, NEWSIZE, EFST_ALLOC_MALLOC, __FILE__, __LINE__)
#define free(BASEADR)               _eff_free(BASEADR, EFST_ALLOC_MALLOC)

#else /* EF_NO_LEAKDETECTION */

void * _eff_malloc(size_t size);
void * _eff_calloc(size_t elemCount, size_t elemSize);
void * _eff_realloc(void * baseAdr, size_t newSize);
void   _eff_free(void * baseAdr);

#define EF_newFrame() do { } while(0)
#define EF_delFrame() do { } while(0)

#endif /* EF_NO_LEAKDETECTION */

/* some special assert */
void _eff_assert(const char * exprstr, const char * filename, int lineno);

#define EF_ASSERT(EXPR)    (  (EXPR) || ( _eff_assert(#EXPR, __FILE__, __LINE__), 0 )  )


/*
 * protection of functions return address
 */
#ifdef __GNUC__

#define EF_FN_PROT_START      const void * EF_RET_ADDR = __builtin_return_address(0); {

#define EF_FN_PROT_END        } EF_ASSERT( __builtin_return_address(0) == EF_RET_ADDR );

#define EF_FN_PROT_RET(EXPR)  do {  EF_ASSERT( __builtin_return_address(0) == EF_RET_ADDR );  return( EXPR ); }  while (0)
#define EF_FN_PROT_RET_VOID() do {  EF_ASSERT( __builtin_return_address(0) == EF_RET_ADDR );  return;         }  while (0)


#else

#define EF_FN_PROT_START      int aiEF_PROT[ 4 ] = { 'E', 'F', 'P', 'R' }; {

#define EF_FN_PROT_END        } EF_ASSERT( 'E'==aiEF_PROT[0] && 'F'==aiEF_PROT[1] && 'P'==aiEF_PROT[2] && 'R'==aiEF_PROT[3] );

#define EF_FN_PROT_RET(EXPR)  do {  EF_ASSERT( 'E'==aiEF_PROT[0] && 'F'==aiEF_PROT[1] && 'P'==aiEF_PROT[2] && 'R'==aiEF_PROT[3] );  return( EXPR ); }  while (0)
#define EF_FN_PROT_RET_VOID() do {  EF_ASSERT( 'E'==aiEF_PROT[0] && 'F'==aiEF_PROT[1] && 'P'==aiEF_PROT[2] && 'R'==aiEF_PROT[3] );  return;         }  while (0)

#endif

/* declaration of an already defined array to enable checking at every reference
 * when using CA_REF()
 */
#define CA_DECLARE(NAME,SIZE) \
const unsigned long NAME ## _checkedsize = (SIZE); \
unsigned long NAME ## _checkedidx

/* definition of a checked array adds definitions for its size and an extra temporary.
 * every array gets its own temporary to avoid problems with threading
 * a global temporary would have.
 */
#define CA_DEFINE(TYPE,NAME,SIZE)  TYPE NAME[SIZE]; CA_DECLARE(NAME,SIZE)

/* every access to a checked array is preceded an assert() on the index;
 * the index parameter is stored to a temporary to avoid double execution of index,
 * when index contains f.e. a "++".
 */
#define CA_REF(NAME,INDEX) \
NAME[ EF_ASSERT( (NAME ## _checkedidx = (INDEX)) < NAME ## _checkedsize ), NAME ## _checkedidx ]


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* end ifdef EF_NO_EFENCE */

==> efence_2_4_13.tar.gz: efence/efence.h <==

/*
 * Electric Fence - Red-Zone memory allocator.
 * Copyright (C) 1987-1999 Bruce Perens <bruce@perens.com>
 * Copyright (C) 2002-2005 Hayati Ayguen <h_ayguen@web.de>, Procitec GmbH
 * License: GNU GPL (GNU General Public License, see COPYING-GPL)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * FILE CONTENTS:
 * header file for inclusion from YOUR application code
 */

/*
 * #include <stdlib.h>
 *
 * You must include <stdlib.h> before including <efence.h> !
 *
 */

/* for enabling inclusion of efence.h after inclusion of efencint.h */
/* remove previous definitions */
#include "noefence.h"

#ifndef _EFENCE_CONFIG_H_
#include "efence_config.h"
#endif


#ifdef EF_NO_EFENCE

/* enable these macros even in release code, but do nothing */
#define EF_newFrame()               do { } while(0)
#define EF_delFrame()               do { } while(0)

#define EF_ASSERT(EXPR)             do { } while(0)

#define CA_DECLARE(NAME,SIZE)       do { } while(0)
#define CA_DEFINE(TYPE,NAME,SIZE)   TYPE NAME[SIZE]
#define CA_REF(NAME,INDEX)          NAME[INDEX]

#else /* ifndef EF_NO_EFENCE */

#ifdef __cplusplus
extern "C" {
#endif

/* global Electric Fence variables */
extern int  EF_PROTECT_BELOW;
extern int  EF_ALIGNMENT;
extern int  EF_FILL;
extern struct _EF_Slot * _ef_allocList;


#ifndef _EFENCE_ENUMS
#define _EFENCE_ENUMS

enum _EF_Allocator
{
    EFA_INT_ALLOC
  , EFA_INT_DEALLOC
  , EFA_MALLOC
  , EFA_CALLOC
  , EFA_FREE
  , EFA_MEMALIGN
  , EFA_REALLOC
  , EFA_VALLOC
  , EFA_NEW_ELEM
  , EFA_DEL_ELEM
  , EFA_NEW_ARRAY
  , EFA_DEL_ARRAY
};

#endif /* _EFENCE_ENUMS */


void   _eff_init(void);


#ifndef EF_NO_LEAKDETECTION

void * _eff_allocate(size_t alignment, size_t userSize, int protectBelow, int fillByte, int protectAllocList, enum _EF_Allocator allocator, const char * filename, int lineno);
void   _eff_deallocate(void * baseAdr, int protectAllocList, enum _EF_Allocator allocator, const char * filename, int lineno);

void * _eff_malloc(size_t size, const char * filename, int lineno);
void * _eff_calloc(size_t elemCount, size_t elemSize, const char * filename, int lineno);
void   _eff_free(void * baseAdr, const char * filename, int lineno);
void * _eff_memalign(size_t alignment, size_t userSize, const char * filename, int lineno);
void * _eff_realloc(void * baseAdr, size_t newSize, const char * filename, int lineno);
void * _eff_valloc(size_t size, const char * filename, int lineno);

void  EF_newFrame(void);
void  EF_delFrame(void);

#define malloc(SIZE)                _eff_malloc(SIZE, __FILE__, __LINE__)
#define calloc(ELEMCOUNT, ELEMSIZE) _eff_calloc(ELEMCOUNT, ELEMSIZE, __FILE__, __LINE__)
#define free(BASEADR)               _eff_free(BASEADR, __FILE__, __LINE__)
#define memalign(ALIGNMENT, SIZE)   _eff_memalign(ALIGNMENT, SIZE, __FILE__, __LINE__)
#define realloc(BASEADR, NEWSIZE)   _eff_realloc(BASEADR, NEWSIZE, __FILE__, __LINE__)
#define valloc(SIZE)                _eff_valloc(SIZE, __FILE__, __LINE__)

#else /* EF_NO_LEAKDETECTION */

void * _eff_allocate(size_t alignment, size_t userSize, int protectBelow, int fillByte, int protectAllocList, enum _EF_Allocator allocator);
void   _eff_deallocate(void * baseAdr, int protectAllocList, enum _EF_Allocator allocator);

void * _eff_malloc(size_t size);
void * _eff_calloc(size_t elemCount, size_t elemSize);
void   _eff_free(void * baseAdr);
void * _eff_memalign(size_t alignment, size_t userSize);
void * _eff_realloc(void * baseAdr, size_t newSize);
void * _eff_valloc(size_t size);

#define EF_newFrame() do { } while(0)
#define EF_delFrame() do { } while(0)

#endif /* EF_NO_LEAKDETECTION */

/* some special assert */
void _eff_assert(const char * exprstr, const char * filename, int lineno);

#define EF_ASSERT(EXPR)    (  (EXPR) || ( _eff_assert(#EXPR, __FILE__, __LINE__), 0 )  )


/*
 * protection of functions return address
 */
#ifdef __GNUC__

#define EF_FN_PROT_START      const void * EF_RET_ADDR = __builtin_return_address(0); {

#define EF_FN_PROT_END        } EF_ASSERT( __builtin_return_address(0) == EF_RET_ADDR );

#define EF_FN_PROT_RET(EXPR)  do {  EF_ASSERT( __builtin_return_address(0) == EF_RET_ADDR );  return( EXPR ); }  while (0)
#define EF_FN_PROT_RET_VOID() do {  EF_ASSERT( __builtin_return_address(0) == EF_RET_ADDR );  return;         }  while (0)

#else

#define EF_FN_PROT_START      int aiEF_PROT[ 4 ] = { 'E', 'F', 'P', 'R' }; {

#define EF_FN_PROT_END        } EF_ASSERT( 'E'==aiEF_PROT[0] && 'F'==aiEF_PROT[1] && 'P'==aiEF_PROT[2] && 'R'==aiEF_PROT[3] );

#define EF_FN_PROT_RET(EXPR)  do {  EF_ASSERT( 'E'==aiEF_PROT[0] && 'F'==aiEF_PROT[1] && 'P'==aiEF_PROT[2] && 'R'==aiEF_PROT[3] );  return( EXPR ); }  while (0)
#define EF_FN_PROT_RET_VOID() do {  EF_ASSERT( 'E'==aiEF_PROT[0] && 'F'==aiEF_PROT[1] && 'P'==aiEF_PROT[2] && 'R'==aiEF_PROT[3] );  return;         }  while (0)

#endif

/* declaration of an already defined array to enable checking at every reference
 * when using CA_REF()
 */
#define CA_DECLARE(NAME,SIZE) \
const unsigned long NAME ## _checkedsize = (SIZE); \
unsigned long NAME ## _checkedidx

/* definition of a checked array adds definitions for its size and an extra temporary.
 * every array gets its own temporary to avoid problems with threading
 * a global temporary would have.
 */
#define CA_DEFINE(TYPE,NAME,SIZE)  TYPE NAME[SIZE]; CA_DECLARE(NAME,SIZE)

/* every access to a checked array is preceded an assert() on the index;
 * the index parameter is stored to a temporary to avoid double execution of index,
 * when index contains f.e. a "++".
 */
#define CA_REF(NAME,INDEX) \
NAME[ EF_ASSERT( (NAME ## _checkedidx = (INDEX)) < NAME ## _checkedsize ), NAME ## _checkedidx ]


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* end ifdef EF_NO_EFENCE */
